@@ -22,10 +22,10 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
             ## If x is a list we need to check if all elements have the $aic to make the model average.
             any.other.class <- any( sapply(hisse.results, function(x) !inherits(x, what="hisse.states") ) )
             if( any.other.class ) stop("All elements of the list 'x' need to be of class 'hisse.states'.")
-            
+
             any.missing <- any( sapply(hisse.results, function(x) is.null(x$aic) ) )
             if( any.missing ) stop( "If x is a list, then each reconstruction need to have $aic information in order to make the model average." )
-            
+
         }
     } else {
         stop( "x needs to be an object of class 'hisse.states'." )
@@ -37,7 +37,7 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
     type <- match.arg(type, c("fan", "phylogram") )
     if( !is.logical( show.tip.label ) ) stop("show.tip.label needs TRUE or FALSE")
     legend <- match.arg(legend, c("none", "traditional", "tips", "internal", "all") )
-    
+
     ## ############################################
     ## Block to get the parameters from the ... list.
     ## Will also check the parameters to see if they are valid.
@@ -132,7 +132,7 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
     }
     ## ############################################
 
-    
+
     ## Going to change par here. So need to save current par and return to previous at the end. Good practice!
     old.par <- par(no.readonly=T)
     if(is.null(rate.colors)) {
@@ -163,29 +163,29 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
     }
     rate.lims[1] <- rate.lims[1] - lims.percentage.correction*abs(rate.lims[1])
     rate.lims[2] <- rate.lims[2] + lims.percentage.correction*abs(rate.lims[2])
-    
+
     rate.tree <- contMapGivenAnc(tree= tree.to.plot, x=rates.tips, plot=FALSE, anc.states=rates.internal, lims=rate.lims)
     #change colors
     rate.colors <- colorRampPalette(rate.colors, space="Lab")(length(rate.tree$cols))
     rate.tree$cols[] <- rate.colors
     #rate.tree$cols[] <- adjustcolor(rate.tree$cols[], alpha.f=0.3)
     #state.tree <- contMapGivenAnc(tree=hisse.object$phy, x=ConvertToBinaryState(hisse.object$tip.mat, state.0.indices=state.0.indices), plot=FALSE, anc.states=ConvertToBinaryState(hisse.object$node.mat, state.0.indices=state.0.indices))
-    
+
     state.lims <- range(c(states.tips, states.internal))
     state.lims[1] <- state.lims[1] - lims.percentage.correction*abs(state.lims[1])
     state.lims[2] <- state.lims[2] + lims.percentage.correction*abs(state.lims[2])
-    
+
     state.tree <- contMapGivenAnc(tree=tree.to.plot, x=states.tips, plot=FALSE, anc.states=states.internal, lims=state.lims)
     #state.colors <- grey(seq(1,0,length.out=length(state.tree$cols)))
     state.colors <- colorRampPalette(state.colors, space="Lab")(length(rate.tree$cols))
     state.tree$cols[]<- state.colors
-    
+
     par(fig=c(0,1, 0, 1), new=FALSE)
     plot.contMapHisse(A=rate.tree, B=state.tree, lwd.factor=width.factor, fsize=fsize,
     , add=FALSE, lwd=edge.width, type=type, mar=mar, direction="rightwards"
     , offset=NULL, xlim=NULL, ylim=NULL, hold=TRUE, swap.underscore=swap.underscore
     , outline=outline, outline.color=outline.color, show.tiplabels=show.tip.label)
-    
+
     if(legend!="none") {
         par(fig=legend.position, new=TRUE)
         plot(x=c(-0.1,1.1), y=c(-1.5,1.5), xlab="", ylab="", bty="n", type="n", xaxt="n", yaxt="n")
@@ -200,7 +200,7 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
             rates.to.plot <- append(rates.to.plot, rates.internal)
             states.to.plot <- append(states.to.plot, states.internal)
         }
-        
+
         if(legend.kernel.rates=="auto") {
             if(length(unique(rates.to.plot))<=4) {
                 legend.kernel.rates <- "hist"
@@ -215,7 +215,7 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
                 legend.kernel.states <- "rectangular"
             }
         }
-        
+
         rates.density <- GetNormalizedDensityPlot(rates.to.plot, rate.lims, legend.kernel.rates)
         states.density <- GetNormalizedDensityPlot(states.to.plot, state.lims, legend.kernel.states)
         states.density$y <- (-1) * states.density$y #so it gets drawn below the other one
@@ -236,7 +236,7 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
         # rates.density$y <- rates.density$y/max(rates.density$y)
         # states.density$y <- (-1) * states.density$y/max(states.density$y)
         # states.density$x <- (states.density$x - state.lims[1]) / (state.lims[2]-state.lims[1]) #so it goes from zero to one
-        
+
         # if(legend=="rect") {
         # rates.density$y <- rep(1, length(rates.density$y))
         # states.density$y <- rep(-1, length(states.density$y))
@@ -247,16 +247,16 @@ plot.hisse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
         text(x=1, y=1.2, labels=format(rate.lims[2], digits=2), cex=legend.cex)
         text(x=0.5, y=1.2, labels=rate.param, cex=legend.cex)
         #		lines(rates.density$x, rates.density$y, lwd=0.5, col="gray")
-        
+
         segments(x0=states.density$x, y0=rep(0, length(states.density$y)), y1=states.density$y, col=state.colors[1+as.integer(round((length(state.colors)-1)* states.density$x))], lwd=ifelse(legend.kernel.states=="hist",4,1))
         text(x=0, y=-1.2, labels="0", cex=legend.cex)
         text(x=1, y=-1.2, labels="1", cex=legend.cex)
         text(x=0.5, y=-1.2, labels="State", cex=legend.cex)
         #		lines(states.density$x, states.density$y, lwd=0.5, col="gray")
         #		lines(rates.density$x, 0*rates.density$y, lwd=0.5, col="gray")
-        
+
     }
-    
+
     ## Return par to the previous state.
     par( old.par )
     return(list(rate.tree=rate.tree, state.tree=state.tree))
@@ -281,7 +281,7 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
             if( any.other.class ) stop("All elements of the list 'x' need to be of class 'hisse.geosse.states'.")
             any.missing <- any( sapply(hisse.results, function(x) is.null(x$aic) ) )
             if( any.missing ) stop( "If x is a list, then each reconstruction need to have $aic information in order to make the model average." )
-            
+
         }
     } else {
         stop( "x needs to be an object of class 'hisse.geosse.states'." )
@@ -294,7 +294,7 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
     if( !is.logical( show.tip.label ) ) stop("show.tip.label needs TRUE or FALSE")
     if( !is.numeric( fsize ) ) stop("fsize needs a numeric value")
     if( !is.logical( legend ) ) stop("legend needs TRUE or FALSE")
-    
+
     ## ############################################
     ## Block to get the parameters from the ... list.
     ## Will also check the parameters to see if they are valid.
@@ -383,10 +383,10 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
         legend.bg <- "cornsilk3"
     }
     ## ############################################
-    
+
     ## Going to change par here. So need to save current par and return to previous at the end. Good practice!
     old.par <- par(no.readonly=T)
-    
+
     ## This sets the coordinates of the figure and facilitate localization.
     par(fig=c(0, 1, 0, 1), new=FALSE)
     if(is.null(rate.colors)) {
@@ -418,12 +418,12 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
     }
     rate.lims[1] <- rate.lims[1] - lims.percentage.correction*abs(rate.lims[1])
     rate.lims[2] <- rate.lims[2] + lims.percentage.correction*abs(rate.lims[2])
-    
+
     rate.tree <- contMapGivenAnc(tree= tree.to.plot, x=rates.tips, plot=FALSE, anc.states=rates.internal, lims=rate.lims)
     ## change colors
     rate.colors <- colorRampPalette(rate.colors, space="Lab")(length(rate.tree$cols))
     rate.tree$cols[] <- rate.colors
-    
+
     states.tips.tmp <- rowSums(states.tips %*% c(0,1,2))
     names(states.tips.tmp) <- 1:length(states.tips.tmp)
     states.internal.tmp <- rowSums(states.internal %*% c(0,1,2))
@@ -434,7 +434,7 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
     state.tree <- contMapGivenAnc(tree=tree.to.plot, x=states.tips.tmp, plot=FALSE, anc.states=states.internal.tmp, lims=state.lims)
     state.colors <- colorRampPalette(state.colors, space="Lab")(length(rate.tree$cols))
     state.tree$cols[]<- state.colors
-    
+
     ## Make the plot.
     ## This is NOT using the phytools version.
     ## This is a special plotting function for HiSSE.
@@ -442,7 +442,7 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
     , add=FALSE, lwd=edge.width, type=type, mar=mar, direction="rightwards"
     , offset=NULL, xlim=NULL, ylim=NULL, hold=TRUE, swap.underscore=swap.underscore
     , outline=outline, outline.color=outline.color, show.tiplabels=show.tip.label)
-    
+
     ## Need to make the legend now:
     ## Supporting only the plot of the rates in this version:
     if( legend ){
@@ -454,7 +454,7 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
                 legend.kernel <- "rectangular"
             }
         }
-        
+
         ## Make the plot:
         par(fig=legend.position, new=TRUE)
         plot(x=c(-0.1, 1.1), y=c(0, 1.5), xlab="", ylab="", bty="n", type="n", xaxt="n", yaxt="n")
@@ -468,7 +468,7 @@ plot.geohisse.states <- function(x, rate.param = "net.div", type = "fan", show.t
         text(x=1, y=1.2, labels=format(rate.lims[2], digits=2), cex=legend.cex)
         text(x=0.5, y=1.2, labels=rate.param, cex=legend.cex)
     }
-    
+
     ## Return par to the previous state.
     par( old.par )
     ## The tiplables for the individual trees will be strange. But this seems fine.
@@ -493,7 +493,7 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
             if( any.other.class ) stop("All elements of the list 'x' need to be of class 'muhisse.states'.")
             any.missing <- any( sapply(hisse.results, function(x) is.null(x$aic) ) )
             if( any.missing ) stop( "If x is a list, then each reconstruction need to have $aic information in order to make the model average." )
-            
+
         }
     } else {
         stop( "x needs to be an object of class 'muhisse.states'." )
@@ -506,7 +506,7 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
     if( !is.logical( show.tip.label ) ) stop("show.tip.label needs TRUE or FALSE")
     if( !is.numeric( fsize ) ) stop("fsize needs a numeric value")
     if( !is.logical( legend ) ) stop("legend needs TRUE or FALSE")
-    
+
     ## ############################################
     ## Block to get the parameters from the ... list.
     ## Will also check the parameters to see if they are valid.
@@ -595,10 +595,10 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
         legend.bg <- "cornsilk3"
     }
     ## ############################################
-    
+
     ## Going to change par here. So need to save current par and return to previous at the end. Good practice!
     old.par <- par(no.readonly=T)
-    
+
     ## This sets the coordinates of the figure and facilitate localization.
     par(fig=c(0, 1, 0, 1), new=FALSE)
     if(is.null(rate.colors)) {
@@ -630,12 +630,12 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
     }
     rate.lims[1] <- rate.lims[1] - lims.percentage.correction*abs(rate.lims[1])
     rate.lims[2] <- rate.lims[2] + lims.percentage.correction*abs(rate.lims[2])
-    
+
     rate.tree <- contMapGivenAnc(tree= tree.to.plot, x=rates.tips, plot=FALSE, anc.states=rates.internal, lims=rate.lims)
     ## change colors
     rate.colors <- colorRampPalette(rate.colors, space="Lab")(length(rate.tree$cols))
     rate.tree$cols[] <- rate.colors
-    
+
     states.tips.tmp <- rowSums(states.tips %*% c(1,2,3,4))
     names(states.tips.tmp) <- 1:length(states.tips.tmp)
     states.internal.tmp <- rowSums(states.internal %*% c(1,2,3,4))
@@ -646,7 +646,7 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
     state.tree <- contMapGivenAnc(tree=tree.to.plot, x=states.tips.tmp, plot=FALSE, anc.states=states.internal.tmp, lims=state.lims)
     state.colors <- colorRampPalette(state.colors, space="Lab")(length(rate.tree$cols))
     state.tree$cols[]<- state.colors
-    
+
     ## Make the plot.
     ## This is NOT using the phytools version.
     ## This is a special plotting function for HiSSE.
@@ -654,7 +654,7 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
     , add=FALSE, lwd=edge.width, type=type, mar=mar, direction="rightwards"
     , offset=NULL, xlim=NULL, ylim=NULL, hold=TRUE, swap.underscore=swap.underscore
     , outline=outline, outline.color=outline.color, show.tiplabels=show.tip.label)
-    
+
     ## Need to make the legend now:
     ## Supporting only the plot of the rates in this version:
     if( legend ){
@@ -666,7 +666,7 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
                 legend.kernel <- "rectangular"
             }
         }
-        
+
         ## Make the plot:
         par(fig=legend.position, new=TRUE)
         plot(x=c(-0.1, 1.1), y=c(0, 1.5), xlab="", ylab="", bty="n", type="n", xaxt="n", yaxt="n")
@@ -680,7 +680,7 @@ plot.muhisse.states <- function(x, rate.param = "net.div", type = "fan", show.ti
         text(x=1, y=1.2, labels=format(rate.lims[2], digits=2), cex=legend.cex)
         text(x=0.5, y=1.2, labels=rate.param, cex=legend.cex)
     }
-    
+
     ## Return par to the previous state.
     par( old.par )
     ## The tiplables for the individual trees will be strange. But this seems fine.
@@ -703,22 +703,22 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
             ## If x is a list we need to check if all elements have the $aic to make the model average.
             any.other.class <- any( sapply(hisse.results, function(x) !inherits(x, what="misse.states") ) )
             if( any.other.class ) stop("All elements of the list 'x' need to be of class 'misse.states'.")
-            
+
             any.missing <- any( sapply(hisse.results, function(x) is.null(x$aic) ) )
             if( any.missing ) stop( "If x is a list, then each reconstruction need to have $aic information in order to make the model average." )
-            
+
         }
     } else {
         stop( "x needs to be an object of class 'misse.states'." )
     }
-    
+
     ## Check the parameters for the functions:
     rate.param <- match.arg(rate.param, c("turnover", "net.div", "speciation", "extinction"
     , "extinction.fraction") )
     type <- match.arg(type, c("fan", "phylogram") )
     if( !is.logical( show.tip.label ) ) stop("show.tip.label needs TRUE or FALSE")
     legend <- match.arg(legend, c("none", "traditional", "tips", "internal", "all") )
-    
+
     ## ############################################
     ## Block to get the parameters from the ... list.
     ## Will also check the parameters to see if they are valid.
@@ -812,8 +812,8 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
         legend.bg <- "cornsilk3"
     }
     ## ############################################
-    
-    
+
+
     ## Going to change par here. So need to save current par and return to previous at the end. Good practice!
     old.par <- par(no.readonly=T)
     if(is.null(rate.colors)) {
@@ -842,29 +842,29 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
     }
     rate.lims[1] <- rate.lims[1] - lims.percentage.correction*abs(rate.lims[1])
     rate.lims[2] <- rate.lims[2] + lims.percentage.correction*abs(rate.lims[2])
-    
+
     rate.tree <- contMapGivenAnc(tree= tree.to.plot, x=rates.tips, plot=FALSE, anc.states=rates.internal, lims=rate.lims)
     #change colors
     rate.colors <- colorRampPalette(rate.colors, space="Lab")(length(rate.tree$cols))
     rate.tree$cols[] <- rate.colors
     #rate.tree$cols[] <- adjustcolor(rate.tree$cols[], alpha.f=0.3)
     #state.tree <- contMapGivenAnc(tree=hisse.object$phy, x=ConvertToBinaryState(hisse.object$tip.mat, state.0.indices=state.0.indices), plot=FALSE, anc.states=ConvertToBinaryState(hisse.object$node.mat, state.0.indices=state.0.indices))
-    
+
     #state.lims <- range(c(states.tips, states.internal))
     #state.lims[1] <- state.lims[1] - lims.percentage.correction*abs(state.lims[1])
     #state.lims[2] <- state.lims[2] + lims.percentage.correction*abs(state.lims[2])
-    
+
     #state.tree <- contMapGivenAnc(tree=tree.to.plot, x=states.tips, plot=FALSE, anc.states=states.internal, lims=state.lims)
     #state.colors <- grey(seq(1,0,length.out=length(state.tree$cols)))
     #state.colors <- colorRampPalette(state.colors, space="Lab")(length(rate.tree$cols))
     #state.tree$cols[]<- state.colors
-    
+
     par(fig=c(0,1, 0, 1), new=FALSE)
     plot.contMapHisse(A=rate.tree, B=rate.tree, lwd.factor=width.factor, fsize=fsize,
     , add=FALSE, lwd=edge.width, type=type, mar=mar, direction="rightwards"
     , offset=NULL, xlim=NULL, ylim=NULL, hold=TRUE, swap.underscore=swap.underscore
     , outline=outline, outline.color=outline.color, show.tiplabels=show.tip.label)
-    
+
     if(legend!="none") {
         par(fig=legend.position, new=TRUE)
         plot(x=c(-0.1,1.1), y=c(-1.5,1.5), xlab="", ylab="", bty="n", type="n", xaxt="n", yaxt="n")
@@ -879,7 +879,7 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
             rates.to.plot <- append(rates.to.plot, rates.internal)
             states.to.plot <- append(states.to.plot, states.internal)
         }
-        
+
         if(legend.kernel.rates=="auto") {
             if(length(unique(rates.to.plot))<=4) {
                 legend.kernel.rates <- "hist"
@@ -894,7 +894,7 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
                 legend.kernel.states <- "rectangular"
             }
         }
-        
+
         rates.density <- GetNormalizedDensityPlot(rates.to.plot, rate.lims, legend.kernel.rates)
         #states.density <- GetNormalizedDensityPlot(states.to.plot, state.lims, legend.kernel.states)
         #states.density$y <- (-1) * states.density$y #so it gets drawn below the other one
@@ -915,7 +915,7 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
         # rates.density$y <- rates.density$y/max(rates.density$y)
         # states.density$y <- (-1) * states.density$y/max(states.density$y)
         # states.density$x <- (states.density$x - state.lims[1]) / (state.lims[2]-state.lims[1]) #so it goes from zero to one
-        
+
         # if(legend=="rect") {
         # rates.density$y <- rep(1, length(rates.density$y))
         # states.density$y <- rep(-1, length(states.density$y))
@@ -926,16 +926,16 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
         text(x=1, y=1.2, labels=format(rate.lims[2], digits=2), cex=legend.cex)
         text(x=0.5, y=1.2, labels=rate.param, cex=legend.cex)
         #		lines(rates.density$x, rates.density$y, lwd=0.5, col="gray")
-        
+
         #segments(x0=states.density$x, y0=rep(0, length(states.density$y)), y1=states.density$y, col=state.colors[1+as.integer(round((length(state.colors)-1)* states.density$x))], lwd=ifelse(legend.kernel.states=="hist",4,1))
         #text(x=0, y=-1.2, labels="0", cex=legend.cex)
         #text(x=1, y=-1.2, labels="1", cex=legend.cex)
         #text(x=0.5, y=-1.2, labels="State", cex=legend.cex)
         #		lines(states.density$x, states.density$y, lwd=0.5, col="gray")
         #		lines(rates.density$x, 0*rates.density$y, lwd=0.5, col="gray")
-        
+
     }
-    
+
     ## Return par to the previous state.
     par( old.par )
     return(list(rate.tree=rate.tree))
@@ -944,7 +944,7 @@ plot.misse.states <- function(x, rate.param = "net.div", type = "fan", show.tip.
 
 GetNormalizedDensityPlot <- function(x, limits, kernel, min.breaks=100) {
     x.density <- c()
-    
+
     if (kernel=="hist") {
         x.density<-hist(x, breaks=seq(from= limits[1], to= limits[2], length.out = max(min.breaks,nclass.Sturges(x)+2)), plot=FALSE)
         x.density$x <- x.density$mid
@@ -1125,7 +1125,7 @@ ConvertManyToRate <- function(hisse.results, rate.param, which.element, AIC.weig
 CheckReconBounds <- function(x, n.models, AIC.weights, bound.par.matrix){
     ## Check if every column of the matrices in the list x is within the bounds set to the each of the parameters.
     ## Drop all the models that are not. Models are the columns in each of the matrices in the list x.
-    
+
     check.keep.mod <- matrix(nrow = 5, ncol = n.models)
     for( i in 1:5 ){
         lim.range <- bound.par.matrix[i, ]
@@ -1186,8 +1186,11 @@ ConvertManyToMultiState <- function(hisse.results, which.element, AIC.weights=NU
     return(final.results)
 }
 
-GetAICWeights <- function(hisse.results) {
-    AIC.vector <- sapply(hisse.results, "[[", "aic")
+GetAICWeights <- function(hisse.results, criterion="aic") {
+    if(class(hisse.results)=="misse.states") {
+        hisse.results <- list(hisse.results)
+    }
+    AIC.vector <- sapply(hisse.results, "[[", criterion)
     delta.AIC.vector <- AIC.vector - min(AIC.vector)
     rel.likelihood <- exp(-0.5 * delta.AIC.vector)
     AIC.weight.vector <- rel.likelihood / sum(rel.likelihood)
